@@ -21,12 +21,9 @@
  *   Emmanuele Bassi  <ebassi@linux.intel.com>
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include <string.h>
-
 #include <glib.h>
 
 #include "json-types-private.h"
@@ -35,9 +32,10 @@
  * SECTION:json-object
  * @short_description: a JSON object representation
  *
- * #JsonObject is the representation of the object type inside JSON. It contains
- * #JsonNode<!-- -->s, which may contain fundamental types, arrays or other
- * objects. Each member of an object is accessed using its name.
+ * #JsonObject is the representation of the object type inside JSON. It
+ * contains #JsonNodes, which may contain fundamental types, arrays or other
+ * objects; each node inside an object, or "member", is accessed using a
+ * unique string, or "name".
  *
  * Since objects can be expensive, they are reference counted. You can control
  * the lifetime of a #JsonObject using json_object_ref() and json_object_unref().
@@ -52,24 +50,24 @@
 G_DEFINE_BOXED_TYPE (JsonObject, json_object, json_object_ref, json_object_unref);
 
 /**
- * json_object_new:
+ * json_object_new: (constructor)
  * 
  * Creates a new #JsonObject, an JSON object type representation.
  *
- * Return value: the newly created #JsonObject
+ * Return value: (transfer full): the newly created #JsonObject
  */
 JsonObject *
 json_object_new (void)
 {
   JsonObject *object;
 
-  object = g_slice_new (JsonObject);
+  object = g_slice_new0 (JsonObject);
   
   object->ref_count = 1;
   object->members = g_hash_table_new_full (g_str_hash, g_str_equal,
                                            g_free,
-                                           (GDestroyNotify) json_node_free);
-  object->members_ordered = NULL;
+                                           (GDestroyNotify) json_node_unref);
+  g_queue_init (&object->members_ordered);
 
   return object;
 }
@@ -80,7 +78,7 @@ json_object_new (void)
  *
  * Increase by one the reference count of a #JsonObject.
  *
- * Return value: the passed #JsonObject, with the reference count
+ * Return value: (transfer none): the passed #JsonObject, with the reference count
  *   increased by one.
  */
 JsonObject *
@@ -89,7 +87,7 @@ json_object_ref (JsonObject *object)
   g_return_val_if_fail (object != NULL, NULL);
   g_return_val_if_fail (object->ref_count > 0, NULL);
 
-  g_atomic_int_add (&object->ref_count, 1);
+  object->ref_count++;
 
   return object;
 }
@@ -108,17 +106,68 @@ json_object_unref (JsonObject *object)
   g_return_if_fail (object != NULL);
   g_return_if_fail (object->ref_count > 0);
 
-  if (g_atomic_int_dec_and_test (&object->ref_count))
+  if (--object->ref_count == 0)
     {
-      g_list_free (object->members_ordered);
+      g_queue_clear (&object->members_ordered);
       g_hash_table_destroy (object->members);
-      object->members_ordered = NULL;
       object->members = NULL;
 
       g_slice_free (JsonObject, object);
     }
 }
 
+/**
+ * json_object_seal:
+ * @object: a #JsonObject
+ *
+ * Seals the #JsonObject, making it immutable to further changes. This will
+ * recursively seal all members of the object too.
+ *
+ * If the @object is already immutable, this is a no-op.
+ *
+ * Since: 1.2
+ */
+void
+json_object_seal (JsonObject *object)
+{
+  JsonObjectIter iter;
+  JsonNode *node;
+
+  g_return_if_fail (object != NULL);
+  g_return_if_fail (object->ref_count > 0);
+
+  if (object->immutable)
+    return;
+
+  /* Propagate to all members. */
+  json_object_iter_init (&iter, object);
+
+  while (json_object_iter_next (&iter, NULL, &node))
+    json_node_seal (node);
+
+  object->immutable_hash = json_object_hash (object);
+  object->immutable = TRUE;
+}
+
+/**
+ * json_object_is_immutable:
+ * @object: a #JsonObject
+ *
+ * Check whether the given @object has been marked as immutable by calling
+ * json_object_seal() on it.
+ *
+ * Since: 1.2
+ * Returns: %TRUE if the @object is immutable
+ */
+gboolean
+json_object_is_immutable (JsonObject *object)
+{
+  g_return_val_if_fail (object != NULL, FALSE);
+  g_return_val_if_fail (object->ref_count > 0, FALSE);
+
+  return object->immutable;
+}
+
 static inline void
 object_set_member_internal (JsonObject  *object,
                             const gchar *member_name,
@@ -127,7 +176,7 @@ object_set_member_internal (JsonObject  *object,
   gchar *name = g_strdup (member_name);
 
   if (g_hash_table_lookup (object->members, name) == NULL)
-    object->members_ordered = g_list_prepend (object->members_ordered, name);
+    g_queue_push_tail (&object->members_ordered, name);
   else
     {
       GList *l;
@@ -136,7 +185,7 @@ object_set_member_internal (JsonObject  *object,
        * pointer to its name, to avoid keeping invalid pointers
        * once we replace the key in the hash table
        */
-      l = g_list_find_custom (object->members_ordered, name, (GCompareFunc) strcmp);
+      l = g_queue_find_custom (&object->members_ordered, name, (GCompareFunc) strcmp);
       if (l != NULL)
         l->data = name;
     }
@@ -424,22 +473,27 @@ json_object_set_object_member (JsonObject  *object,
  * Retrieves all the names of the members of a #JsonObject. You can
  * obtain the value for each member using json_object_get_member().
  *
- * Return value: (element-type utf8) (transfer container): a #GList
- *   of member names. The content of the list is owned by the #JsonObject
- *   and should never be modified or freed. When you have finished using
- *   the returned list, use g_list_free() to free the resources it has
- *   allocated.
+ * Return value: (element-type utf8) (transfer container) (nullable): a
+ *   #GList of member names, or %NULL. The content of the list is owned
+ *   by the #JsonObject and should never be modified or freed. When you
+ *   have finished using the returned list, use g_list_free() to free
+ *   the resources it has allocated.
  */
 GList *
 json_object_get_members (JsonObject *object)
 {
-  GList *copy;
-
   g_return_val_if_fail (object != NULL, NULL);
 
-  copy = g_list_copy (object->members_ordered);
+  return g_list_copy (object->members_ordered.head);
+}
 
-  return g_list_reverse (copy);
+
+GQueue *
+json_object_get_members_internal (JsonObject *object)
+{
+  g_return_val_if_fail (object != NULL, NULL);
+
+  return &object->members_ordered;
 }
 
 /**
@@ -448,10 +502,11 @@ json_object_get_members (JsonObject *object)
  *
  * Retrieves all the values of the members of a #JsonObject.
  *
- * Return value: (element-type JsonNode) (transfer container): a #GList of
- *   #JsonNode<!-- -->s. The content of the list is owned by the #JsonObject
- *   and should never be modified or freed. When you have finished using the
- *   returned list, use g_list_free() to free the resources it has allocated.
+ * Return value: (element-type JsonNode) (transfer container) (nullable): a
+ *   #GList of #JsonNodes, or %NULL. The content of the list is owned by the
+ *   #JsonObject and should never be modified or freed. When you have finished
+ *   using the returned list, use g_list_free() to free the resources it has
+ *   allocated.
  */
 GList *
 json_object_get_values (JsonObject *object)
@@ -461,7 +516,7 @@ json_object_get_values (JsonObject *object)
   g_return_val_if_fail (object != NULL, NULL);
 
   values = NULL;
-  for (l = object->members_ordered; l != NULL; l = l->next)
+  for (l = object->members_ordered.tail; l != NULL; l = l->prev)
     values = g_list_prepend (values, g_hash_table_lookup (object->members, l->data));
 
   return values;
@@ -475,8 +530,8 @@ json_object_get_values (JsonObject *object)
  * Retrieves a copy of the #JsonNode containing the value of @member_name
  * inside a #JsonObject
  *
- * Return value: (transfer full): a copy of the node for the requested
- *   object member or %NULL. Use json_node_free() when done.
+ * Return value: (transfer full) (nullable): a copy of the node for the
+ *   requested object member or %NULL. Use json_node_unref() when done.
  *
  * Since: 0.6
  */
@@ -511,8 +566,8 @@ object_get_member_internal (JsonObject  *object,
  * Retrieves the #JsonNode containing the value of @member_name inside
  * a #JsonObject.
  *
- * Return value: (transfer none): a pointer to the node for the requested object
- *   member, or %NULL
+ * Return value: (transfer none) (nullable): a pointer to the node for the
+ *   requested object member, or %NULL
  */
 JsonNode *
 json_object_get_member (JsonObject  *object,
@@ -724,11 +779,13 @@ json_object_get_array_member (JsonObject  *object,
  * @member_name: the name of the member
  *
  * Convenience function that retrieves the object
- * stored in @member_name of @object
+ * stored in @member_name of @object. It is an error to specify a @member_name
+ * which does not exist.
  *
  * See also: json_object_get_member()
  *
- * Return value: (transfer none): the object inside the object's member
+ * Return value: (transfer none) (nullable): the object inside the object’s
+ *    member, or %NULL if the value for the member is `null`
  *
  * Since: 0.8
  */
@@ -802,13 +859,13 @@ json_object_remove_member (JsonObject  *object,
   g_return_if_fail (object != NULL);
   g_return_if_fail (member_name != NULL);
 
-  for (l = object->members_ordered; l != NULL; l = l->next)
+  for (l = object->members_ordered.head; l != NULL; l = l->next)
     {
       const gchar *name = l->data;
 
       if (g_strcmp0 (name, member_name) == 0)
         {
-          object->members_ordered = g_list_delete_link (object->members_ordered, l);
+          g_queue_delete_link (&object->members_ordered, l);
           break;
         }
     }
@@ -836,14 +893,12 @@ json_object_foreach_member (JsonObject        *object,
                             JsonObjectForeach  func,
                             gpointer           data)
 {
-  GList *members, *l;
+  GList *l;
 
   g_return_if_fail (object != NULL);
   g_return_if_fail (func != NULL);
 
-  /* the list is stored in reverse order to have constant time additions */
-  members = g_list_last (object->members_ordered);
-  for (l = members; l != NULL; l = l->prev)
+  for (l = object->members_ordered.head; l != NULL; l = l->next)
     {
       const gchar *member_name = l->data;
       JsonNode *member_node = g_hash_table_lookup (object->members, member_name);
@@ -851,3 +906,170 @@ json_object_foreach_member (JsonObject        *object,
       func (object, member_name, member_node, data);
     }
 }
+
+/**
+ * json_object_hash:
+ * @key: (type JsonObject): a JSON object to hash
+ *
+ * Calculate a hash value for the given @key (a #JsonObject).
+ *
+ * The hash is calculated over the object and all its members, recursively. If
+ * the object is immutable, this is a fast operation; otherwise, it scales
+ * proportionally with the number of members in the object.
+ *
+ * Returns: hash value for @key
+ * Since: 1.2
+ */
+guint
+json_object_hash (gconstpointer key)
+{
+  JsonObject *object = (JsonObject *) key;
+  guint hash = 0;
+  JsonObjectIter iter;
+  const gchar *member_name;
+  JsonNode *node;
+
+  g_return_val_if_fail (object != NULL, 0);
+
+  /* If the object is immutable, use the cached hash. */
+  if (object->immutable)
+    return object->immutable_hash;
+
+  /* Otherwise, calculate from scratch. */
+  json_object_iter_init (&iter, object);
+
+  while (json_object_iter_next (&iter, &member_name, &node))
+    hash ^= (json_string_hash (member_name) ^ json_node_hash (node));
+
+  return hash;
+}
+
+/**
+ * json_object_equal:
+ * @a: (type JsonObject): a JSON object
+ * @b: (type JsonObject): another JSON object
+ *
+ * Check whether @a and @b are equal #JsonObjects, meaning they have the same
+ * set of members, and the values of corresponding members are equal.
+ *
+ * Returns: %TRUE if @a and @b are equal; %FALSE otherwise
+ * Since: 1.2
+ */
+gboolean
+json_object_equal (gconstpointer  a,
+                   gconstpointer  b)
+{
+  JsonObject *object_a, *object_b;
+  guint size_a, size_b;
+  JsonObjectIter iter_a;
+  JsonNode *child_a, *child_b;  /* unowned */
+  const gchar *member_name;
+
+  object_a = (JsonObject *) a;
+  object_b = (JsonObject *) b;
+
+  /* Identity comparison. */
+  if (object_a == object_b)
+    return TRUE;
+
+  /* Check sizes. */
+  size_a = json_object_get_size (object_a);
+  size_b = json_object_get_size (object_b);
+
+  if (size_a != size_b)
+    return FALSE;
+
+  /* Check member names and values. Check the member names first
+   * to avoid expensive recursive value comparisons which might
+   * be unnecessary. */
+  json_object_iter_init (&iter_a, object_a);
+
+  while (json_object_iter_next (&iter_a, &member_name, NULL))
+    {
+      if (!json_object_has_member (object_b, member_name))
+        return FALSE;
+    }
+
+  json_object_iter_init (&iter_a, object_a);
+
+  while (json_object_iter_next (&iter_a, &member_name, &child_a))
+    {
+      child_b = json_object_get_member (object_b, member_name);
+
+      if (!json_node_equal (child_a, child_b))
+        return FALSE;
+    }
+
+  return TRUE;
+}
+
+/**
+ * json_object_iter_init:
+ * @iter: an uninitialised #JsonObjectIter
+ * @object: the #JsonObject to iterate over
+ *
+ * Initialise the @iter and associate it with @object.
+ *
+ * |[<!-- language="C" -->
+ * JsonObjectIter iter;
+ * const gchar *member_name;
+ * JsonNode *member_node;
+ *
+ * json_object_iter_init (&iter, some_object);
+ * while (json_object_iter_next (&iter, &member_name, &member_node))
+ *   {
+ *     // Do something with @member_name and @member_node.
+ *   }
+ * ]|
+ *
+ * Since: 1.2
+ */
+void
+json_object_iter_init (JsonObjectIter  *iter,
+                       JsonObject      *object)
+{
+  JsonObjectIterReal *iter_real = (JsonObjectIterReal *) iter;;
+
+  g_return_if_fail (iter != NULL);
+  g_return_if_fail (object != NULL);
+  g_return_if_fail (object->ref_count > 0);
+
+  iter_real->object = object;
+  g_hash_table_iter_init (&iter_real->members_iter, object->members);
+}
+
+/**
+ * json_object_iter_next:
+ * @iter: a #JsonObjectIter
+ * @member_name: (out callee-allocates) (transfer none) (optional): return
+ *    location for the member name, or %NULL to ignore
+ * @member_node: (out callee-allocates) (transfer none) (optional): return
+ *    location for the member value, or %NULL to ignore
+ *
+ * Advance @iter and retrieve the next member in the object. If the end of the
+ * object is reached, %FALSE is returned and @member_name and @member_node are
+ * set to invalid values. After that point, the @iter is invalid.
+ *
+ * The order in which members are returned by the iterator is undefined. The
+ * iterator is invalidated if its #JsonObject is modified during iteration.
+ *
+ * Returns: %TRUE if @member_name and @member_node are valid; %FALSE if the end
+ *    of the object has been reached
+ *
+ * Since: 1.2
+ */
+gboolean
+json_object_iter_next (JsonObjectIter  *iter,
+                       const gchar    **member_name,
+                       JsonNode       **member_node)
+{
+  JsonObjectIterReal *iter_real = (JsonObjectIterReal *) iter;
+
+  g_return_val_if_fail (iter != NULL, FALSE);
+  g_return_val_if_fail (iter_real->object != NULL, FALSE);
+  g_return_val_if_fail (iter_real->object->ref_count > 0, FALSE);
+
+  return g_hash_table_iter_next (&iter_real->members_iter,
+                                 (gpointer *) member_name,
+                                 (gpointer *) member_node);
+}
