@@ -16,7 +16,7 @@ static const gchar *empty_object = "{}";
 static const gchar *simple_array = "[true,false,null,42,\"foo\"]";
 static const gchar *nested_array = "[true,[false,null],42]";
 
-static const gchar *simple_object = "{\"Bool1\":true,\"Bool2\":false,\"Null\":null,\"Int\":42,\"String\":\"foo\"}";
+static const gchar *simple_object = "{\"Bool1\":true,\"Bool2\":false,\"Null\":null,\"Int\":42,\"\":54,\"String\":\"foo\"}";
 /* taken from the RFC 4627, Examples section */
 static const gchar *nested_object =
 "{"
@@ -209,6 +209,7 @@ test_simple_object (void)
   json_object_set_boolean_member (object, "Bool2", FALSE);
   json_object_set_null_member (object, "Null");
   json_object_set_int_member (object, "Int", 42);
+  json_object_set_int_member (object, "", 54);
   json_object_set_string_member (object, "String", "foo");
 
   json_node_take_object (root, object);
@@ -334,6 +335,27 @@ test_decimal_separator (void)
   json_node_free (node);
 }
 
+
+static void
+test_double_stays_double (void)
+{
+  gchar *str;
+  JsonNode *node = json_node_new (JSON_NODE_VALUE);
+  JsonGenerator *generator = json_generator_new ();
+
+  json_node_set_double (node, 1.0);
+
+  json_generator_set_root (generator, node);
+
+  str = json_generator_to_data (generator, NULL);
+  g_assert_cmpstr (str, ==, "1.0");
+
+  g_free (str);
+  g_object_unref (generator);
+  json_node_free (node);
+}
+
+
 static void
 test_pretty (void)
 {
@@ -426,6 +448,7 @@ main (int   argc,
   g_test_add_func ("/generator/simple-object", test_simple_object);
   g_test_add_func ("/generator/nested-object", test_nested_object);
   g_test_add_func ("/generator/decimal-separator", test_decimal_separator);
+  g_test_add_func ("/generator/double-stays-double", test_double_stays_double);
   g_test_add_func ("/generator/pretty", test_pretty);
 
   for (i = 0; i < G_N_ELEMENTS (string_fixtures); i++)
