@@ -1,9 +1,4 @@
-#include <stdlib.h>
-#include <stdio.h>
-
-#include <glib.h>
-
-#include <json-glib/json-glib.h>
+#include "json-test-utils.h"
 
 static const gchar *test_base_array_data =
 "[ 0, true, null, \"foo\", 3.14, [ false ], { \"bar\" : 42 } ]";
@@ -14,6 +9,10 @@ static const gchar *test_base_object_data =
 static const gchar *test_reader_level_data =
 " { \"list\": { \"181195771\": { \"given_url\": \"http://www.gnome.org/json-glib-test\" } } }";
 
+/* https://bugzilla.gnome.org/show_bug.cgi?id=758580 */
+static const char *test_reader_null_value_data =
+"{ \"v\": null }";
+
 static const gchar *expected_member_name[] = {
   "text",
   "foo",
@@ -74,7 +73,7 @@ test_base_object (void)
   g_assert (json_reader_get_error (reader) == NULL);
 
   json_reader_read_member (reader, "double");
-  g_assert_cmpfloat (json_reader_get_double_value (reader), ==, 42.47);
+  json_assert_fuzzy_equals (json_reader_get_double_value (reader), 42.47, 0.01);
   json_reader_end_element (reader);
 
   g_object_unref (reader);
@@ -134,6 +133,8 @@ test_base_array (void)
   g_assert (json_reader_get_error (reader) == NULL);
 
   g_object_unref (reader);
+  g_object_unref (parser);
+  g_clear_error (&error);
 }
 
 static void
@@ -188,6 +189,29 @@ test_reader_level (void)
   g_clear_object (&parser);
 }
 
+static void
+test_reader_null_value (void)
+{
+  JsonParser *parser = json_parser_new ();
+  JsonReader *reader = json_reader_new (NULL);
+  GError *error = NULL;
+
+  g_test_bug ("758580");
+
+  json_parser_load_from_data (parser, test_reader_null_value_data, -1, &error);
+  g_assert_no_error (error);
+
+  json_reader_set_root (reader, json_parser_get_root (parser));
+
+  json_reader_read_member (reader, "v");
+  g_assert_true (json_reader_is_value (reader));
+  g_assert_no_error (json_reader_get_error (reader));
+  g_assert_nonnull (json_reader_get_value (reader));
+
+  g_object_unref (reader);
+  g_object_unref (parser);
+}
+
 int
 main (int   argc,
       char *argv[])
@@ -198,6 +222,7 @@ main (int   argc,
   g_test_add_func ("/reader/base-array", test_base_array);
   g_test_add_func ("/reader/base-object", test_base_object);
   g_test_add_func ("/reader/level", test_reader_level);
+  g_test_add_func ("/reader/null-value", test_reader_null_value);
 
   return g_test_run ();
 }
